package com.ebaiyihui.family.doctor.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.doctor.basedata.api.vo.DepartmentDetailVo;
import com.doctoruser.api.pojo.base.dto.doctor.QueryPersonnelInfoReq;
import com.doctoruser.api.pojo.base.vo.doctor.PersonnelInfo;
import com.ebaiyihui.family.doctor.common.vo.cloudHis.*;
import com.ebaiyihui.family.doctor.server.common.constants.ProjProperties;
import com.ebaiyihui.family.doctor.server.common.enums.CloudHisAdmissionStatusEnum;
import com.ebaiyihui.family.doctor.server.entity.PatientEntity;
import com.ebaiyihui.family.doctor.server.entity.PatientSignEntity;
import com.ebaiyihui.family.doctor.server.feign.DepartmentFeignClient;
import com.ebaiyihui.family.doctor.server.feign.DoctorInfofeignClient;
import com.ebaiyihui.family.doctor.server.mapper.PatientMapper;
import com.ebaiyihui.family.doctor.server.service.HisTemplateService;
import com.ebaiyihui.family.doctor.server.util.HttpKit;
import com.ebaiyihui.framework.response.BaseResponse;
import com.ebaiyihui.his.pojo.vo.base.FrontRequest;
import com.ebaiyihui.his.pojo.vo.base.FrontResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;


@Service
@Slf4j
public class HisTemplateServiceImpl implements HisTemplateService {

    @Autowired
    private ProjProperties projProperties;

    @Autowired
    private DepartmentFeignClient departmentInfoApi;

    @Autowired
    private DoctorInfofeignClient doctorClient;

    @Autowired
    private PatientMapper patientMapper;


    @Override
    public BaseResponse<Object> registrationAndTreatment(PatientSignEntity patientSign) {

        HisDeptAndDocCodeVo hisDeptAndDocCode = getHisDeptAndDocCode(patientSign.getDeptId().toString(), String.valueOf(patientSign.getDoctorId()));

        //挂号
        AppPointRequestDTO appPointRequestDTO = this.buildAppointmentParam(patientSign, hisDeptAndDocCode);
        log.info("0元复诊挂号入参:{}", JSON.toJSONString(appPointRequestDTO));
        MakeAnAppointmentRequestVO makeAnAppointmentRequestVO = admissionRegistration(appPointRequestDTO);
        if (Objects.isNull(makeAnAppointmentRequestVO) || StringUtils.isEmpty(makeAnAppointmentRequestVO.getOutpatientNo())) {
            return BaseResponse.error("云his挂号失败");
        }
        // 接诊
        log.info("南昌众康医生接诊======");
        if (StringUtils.isEmpty(makeAnAppointmentRequestVO.getOutpatientNo())) {
            return BaseResponse.error("挂号失败,不能接诊");
        }
        AppPointRequestDTO receive = new AppPointRequestDTO();
        receive.setOutpatientNo(makeAnAppointmentRequestVO.getOutpatientNo());
        receive.setStatus(CloudHisAdmissionStatusEnum.TREATNENTED.getValue());
        log.info("复诊接诊入参:{}", JSON.toJSONString(receive));
        FrontResponse<String> hisResponse = updateAppointment(receive);
        if (FrontResponse.FAILED_CODE.equals(hisResponse.getCode())) {
            return BaseResponse.error("挂号失败，无法接诊");
        }
        return BaseResponse.success(makeAnAppointmentRequestVO.getOutpatientNo());
    }

    @Override
    public MakeAnAppointmentRequestVO admissionRegistration(AppPointRequestDTO appPointRequestDTO) {
        String url = projProperties.getAdmissionRegistration();
        FrontRequest<AppPointRequestDTO> request = new FrontRequest<>();
        request.setBody(appPointRequestDTO);
        try {
            log.info("云his=>>新增就诊信息请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse<MakeAnAppointmentRequestVO> frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>新增就诊信息返回值:{}", JSONObject.toJSONString(frontResponse));
            MakeAnAppointmentRequestVO makeAnAppointmentRequestVO = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    MakeAnAppointmentRequestVO.class);
            log.info("云his=>新增就诊记录返回值:{}", JSONObject.toJSONString(makeAnAppointmentRequestVO));
            return makeAnAppointmentRequestVO;
        } catch (Exception e) {
            log.error("新增预约信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<String> updateAppointment(AppPointRequestDTO appPointRequestDTO) {
        String url = projProperties.getAdmissionUpdate();
        FrontRequest<AppPointRequestDTO> request = new FrontRequest<>();
        request.setBody(appPointRequestDTO);
        try {
            log.info("云his=>>更新挂号信息请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>更新挂号信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            String data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    String.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("更新挂号信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<String> saveDiagnostic(DiagnosticReqDTO diagnosticReqDTO) {
        String url = projProperties.getUpdateOutpatientMedicalRecords();
        FrontRequest<DiagnosticReqDTO> request = new FrontRequest<>();
        request.setBody(diagnosticReqDTO);
        try {
            log.info("云his=>>保存诊断信息请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>保存诊断信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            String data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    String.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("保存诊断信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<FeeBillCreationVo> createCharge(FeeBillCreationDto feeBillCreationDto) {
        String url = projProperties.getCreateOrderCharge();
        FrontRequest<FeeBillCreationDto> request = new FrontRequest<>();
        request.setBody(feeBillCreationDto);
        try {
            log.info("云his=>>复诊订单预支付信息请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>复诊订单预支付信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            FeeBillCreationVo data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    FeeBillCreationVo.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("复诊订单预支付信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<PayVo> payment(PayDto payDto) {
        String url = projProperties.getPayment();
        FrontRequest<PayDto> request = new FrontRequest<>();
        request.setBody(payDto);
        try {
            log.info("云his=>复诊订单支付信息请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>复诊订单支付信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            PayVo data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    PayVo.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("复诊订单支付信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<String> archiving(ArchivingRequestDTO archivingRequestDTO) {
        String url = projProperties.getArchiving();
        FrontRequest<ArchivingRequestDTO> request = new FrontRequest<>();
        request.setBody(archivingRequestDTO);
        try {
            log.info("云his=>复诊病历归档请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>复诊病历归档信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            String data = JSON.toJSONString(frontResponse.getBody());
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("复诊病历归档信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<PayRefundVo> refund(PayRefundDto payRefundDto) {
        String url = projProperties.getRefund();
        FrontRequest<PayRefundDto> request = new FrontRequest<>();
        request.setBody(payRefundDto);
        try {
            log.info("云his=>复诊退款请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>复诊退款信息返回值:{}" + JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return null;
            }
            PayRefundVo data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    PayRefundVo.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("复诊退款信息异常:{}", e.getMessage());
        }
        return null;
    }

    @Override
    public FrontResponse<OutpatientTypeVo> checkDocAndDeptAndRegFeeInfo(HisDeptAndDocCodeVo hisDeptAndDocCode) {
        String url = projProperties.getCheckDocAndDeptAndRegFeeInfo();
        FrontRequest<HisDeptAndDocCodeVo> request = new FrontRequest<>();
        request.setBody(hisDeptAndDocCode);
        try {
            log.info("云his=>复诊挂号类型医生部门校验请求url:{},param:{}", url, JSON.toJSONString(request));
            String result = HttpKit.jsonPost(url, JSON.toJSONString(request));
            FrontResponse frontResponse = JSONObject.toJavaObject(JSONObject.parseObject(result), FrontResponse.class);
            log.info("云his=>复诊挂号类型医生部门校验返回值:{}", JSONObject.toJSONString(frontResponse));
            if (!FrontResponse.SUCCESS_CODE.equals(frontResponse.getCode())) {
                return FrontResponse.error(null, FrontResponse.FAILED_CODE, frontResponse.getMessage());
            }
            OutpatientTypeVo data = JSON.parseObject(JSON.toJSONString(frontResponse.getBody()),
                    OutpatientTypeVo.class);
            return FrontResponse.success(null, data);
        } catch (Exception e) {
            log.error("复诊挂号类型医生部门校验异常:{}", e.getMessage());
            return FrontResponse.error(null, FrontResponse.FAILED_CODE, "复诊挂号类型医生部门校验异常");
        }
    }


    @Override
    public HisDeptAndDocCodeVo getHisDeptAndDocCode(String deptId, String doctorId) {

        HisDeptAndDocCodeVo hisDeptAndDocCodeVo = new HisDeptAndDocCodeVo();
        QueryPersonnelInfoReq queryPersonnelInfoReq = new QueryPersonnelInfoReq();
        queryPersonnelInfoReq.setDoctorId(doctorId);
        BaseResponse<PersonnelInfo> personnelInfoBaseResponse = doctorClient.queryPersonnelInfo(queryPersonnelInfoReq);
        if (personnelInfoBaseResponse.isSuccess() && null != personnelInfoBaseResponse.getData()) {
            log.info("his医生code:" + JSON.toJSONString(personnelInfoBaseResponse));
            String emplCode = personnelInfoBaseResponse.getData().getEmplCode();
            hisDeptAndDocCodeVo.setDoctorCode(emplCode);
            hisDeptAndDocCodeVo.setDoctorName(personnelInfoBaseResponse.getData().getDoctorName());
        }

        Long deptIdL = Long.valueOf(deptId);
        BaseResponse<DepartmentDetailVo> departmentDetail = departmentInfoApi.getDepartmentDetail(deptIdL);
        if (departmentDetail.isSuccess() && null != departmentDetail.getData()) {
            log.info("his科室code:" + JSON.toJSONString(departmentDetail));
            String deptCode = departmentDetail.getData().getDeptCode();
            hisDeptAndDocCodeVo.setDeptCode(deptCode);
            hisDeptAndDocCodeVo.setDeptName(departmentDetail.getData().getDeptName());
        }
        return hisDeptAndDocCodeVo;
    }

    private AppPointRequestDTO buildAppointmentParam(PatientSignEntity patientEntity, HisDeptAndDocCodeVo hisDeptAndDocCode) {
        // 查询患者信息
        String patientId = patientEntity.getPatientId();
        PatientEntity patientInfo = patientMapper.selectById(patientId);
        AppPointRequestDTO appPointRequestDTO = new AppPointRequestDTO();
        appPointRequestDTO.setCardType("01");
        appPointRequestDTO.setName(patientEntity.getPatientName());
        appPointRequestDTO.setOutpatientType("1");
        appPointRequestDTO.setDeptId(Integer.valueOf(hisDeptAndDocCode.getDeptCode()));
        appPointRequestDTO.setSex(patientInfo.getGender() + "");
        appPointRequestDTO.setCardNo(patientInfo.getCredNo());
        appPointRequestDTO.setPhone(patientInfo.getPhone());
        appPointRequestDTO.setDoctorName(hisDeptAndDocCode.getDoctorName());
        appPointRequestDTO.setMedicalType("1");
        appPointRequestDTO.setDoctorId(Integer.valueOf(hisDeptAndDocCode.getDoctorCode()));
        appPointRequestDTO.setDeptName(hisDeptAndDocCode.getDeptName());
        appPointRequestDTO.setHospitalName("众康家庭医生");
        return appPointRequestDTO;
    }
}
